package api.facturalo.rest;

/**
 * PARA CONEXIN HTTP A SERVICIO REST
 */
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.URL;
import java.net.URLEncoder;
import java.util.stream.Collectors;

/**
 * PARA MANEJO DE RESPUESTA EN JSON
 */
import com.google.gson.Gson;
import com.google.gson.JsonSyntaxException;

public class ServicioDo {
	protected String url;
	protected static HttpURLConnection urlConn;
	protected URL mUrl;

	public RespuestaTimbrado timbrar(String apikey, String xmlCFDI) {
		RespuestaTimbrado result = new RespuestaTimbrado();
		String resp = "";
		try {
			String operacion = "timbrar";
			// SE ABRE LA CONEXIN
			mUrl = new URL(url+operacion);
			urlConn = (HttpURLConnection) mUrl.openConnection();

			// SE DEFINE LA CODIFICACIN DEL MENSAJE
			String charset = "UTF-8";  // Or in Java 7 and later, use the constant: java.nio.charset.StandardCharsets.UTF_8.name()

			// SE DEFINE EL CUERPO DEL MENSAJE
			String query = String.format("apikey=%s&xmlCFDI=%s",
					URLEncoder.encode(apikey, charset),
				    URLEncoder.encode(xmlCFDI, charset)
				    );
			
			// SE REALIZA LA SOLICITUD
			resp = doRequest(query, charset);
			
			// SE SERIALIZA LA RESPUESTA A UN OBJETO
			result = new Gson().fromJson(resp, RespuestaTimbrado.class);
		} catch (IOException e1) {
			result.setMessage(e1.getMessage());
		} catch (JsonSyntaxException e2) {
			String mensaje = e2.getMessage() + ". " + resp;
			result.setMessage(mensaje);
		}
		
		return result;
	}

	public RespuestaTimbrado timbrarConSello(String apikey, String xmlCFDI, String keyPEM) {
		RespuestaTimbrado result = new RespuestaTimbrado();
		String resp = "";
		try {
			String operacion = "timbrarConSello";
			// SE ABRE LA CONEXIN
			mUrl = new URL(url+operacion);
			urlConn = (HttpURLConnection) mUrl.openConnection();

			// SE DEFINE LA CODIFICACIN DEL MENSAJE
			String charset = "UTF-8";  // Or in Java 7 and later, use the constant: java.nio.charset.StandardCharsets.UTF_8.name()

			// SE DEFINE EL CUERPO DEL MENSAJE
			String query = String.format("apikey=%s&xmlCFDI=%s&keyPEM=%s",
					URLEncoder.encode(apikey, charset),
				    URLEncoder.encode(xmlCFDI, charset),
				    URLEncoder.encode(keyPEM, charset)
				    );
			
			// SE REALIZA LA SOLICITUD
			resp = doRequest(query, charset);
			
			// SE SERIALIZA LA RESPUESTA A UN OBJETO
			result = new Gson().fromJson(resp, RespuestaTimbrado.class);
		} catch (IOException e1) {
			result.setMessage(e1.getMessage());
		} catch (JsonSyntaxException e2) {
			String mensaje = e2.getMessage() + ". " + resp;
			result.setMessage(mensaje);
		}
		
		return result;
	}
	
	public RespuestaCancelar cancelar(String apikey, String keyCSD, String cerCSD, 
			String passCSD, String uuid, String rfcEmisor, String rfcReceptor, 
			String total, String motivo, String folioSustitucion) {
		RespuestaCancelar result = new RespuestaCancelar();
		String resp = "";
		try {
			String operacion = "cancelar";
			// SE ABRE LA CONEXIN
			mUrl = new URL(url+operacion);
			urlConn = (HttpURLConnection) mUrl.openConnection();

			// SE DEFINE LA CODIFICACIN DEL MENSAJE
			String charset = "UTF-8";  // Or in Java 7 and later, use the constant: java.nio.charset.StandardCharsets.UTF_8.name()

			// SE DEFINE EL CUERPO DEL MENSAJE
			String query = String.format("apikey=%s&keyCSD=%s&cerCSD=%s&passCSD=%s&uuid=%s"+
					"&rfcEmisor=%s&rfcReceptor=%s&total=%s&motivo=%s&folioSustitucion=%s",
					URLEncoder.encode(apikey, charset),
					URLEncoder.encode(keyCSD, charset),
					URLEncoder.encode(cerCSD, charset),
					URLEncoder.encode(passCSD, charset),
				    URLEncoder.encode(uuid, charset),
				    URLEncoder.encode(rfcEmisor, charset),
					URLEncoder.encode(rfcReceptor, charset),
					URLEncoder.encode(total, charset),
					URLEncoder.encode(motivo, charset),
				    URLEncoder.encode(folioSustitucion, charset)
				    );
			
			// SE REALIZA LA SOLICITUD
			resp = doRequest(query, charset);
			
			// SE SERIALIZA LA RESPUESTA A UN OBJETO
			result = new Gson().fromJson(resp, RespuestaCancelar.class);
			
			if (!result.getData().equals(null)) {
				String jsondata = new Gson().toJson(result.getData());
				DataCancelar datajson = new Gson().fromJson(jsondata, DataCancelar.class);
				result.setData(datajson.getAcuse());
			}
		} catch (IOException e1) {
			result.setMessage(e1.getMessage());
		} catch (JsonSyntaxException e2) {
			String mensaje = result.getMessage();
						
			if (mensaje == null || mensaje.isEmpty()) {
				mensaje = e2.getMessage() + ". " + resp;
			}
			
			result.setMessage(mensaje);
		}
		
		return result;
	}
	
	public RespuestaCancelar cancelarPFX(String apikey, String filePFX, 
			String passPFX, String uuid, String rfcEmisor, String rfcReceptor, 
			String total, String motivo, String folioSustitucion) {
		RespuestaCancelar result = new RespuestaCancelar();
		String resp = "";
		try {
			String operacion = "cancelarPFX";
			// SE ABRE LA CONEXIN
			mUrl = new URL(url+operacion);
			urlConn = (HttpURLConnection) mUrl.openConnection();

			// SE DEFINE LA CODIFICACIN DEL MENSAJE
			String charset = "UTF-8";  // Or in Java 7 and later, use the constant: java.nio.charset.StandardCharsets.UTF_8.name()

			// SE DEFINE EL CUERPO DEL MENSAJE
			String query = String.format("apikey=%s&filePFX=%s&passPFX=%s&uuid=%s"+
					"&rfcEmisor=%s&rfcReceptor=%s&total=%s&motivo=%s&folioSustitucion=%s",
					URLEncoder.encode(apikey, charset),
					URLEncoder.encode(filePFX, charset),
					URLEncoder.encode(passPFX, charset),
				    URLEncoder.encode(uuid, charset),
				    URLEncoder.encode(rfcEmisor, charset),
					URLEncoder.encode(rfcReceptor, charset),
					URLEncoder.encode(total, charset),
					URLEncoder.encode(motivo, charset),
				    URLEncoder.encode(folioSustitucion, charset)
				    );
			
			// SE REALIZA LA SOLICITUD
			resp = doRequest(query, charset);
			
			// SE SERIALIZA LA RESPUESTA A UN OBJETO
			result = new Gson().fromJson(resp, RespuestaCancelar.class);
			
			if (!result.getData().equals(null)) {
				String jsondata = new Gson().toJson(result.getData());
				DataCancelar datajson = new Gson().fromJson(jsondata, DataCancelar.class);
				result.setData(datajson.getAcuse());
			}
		} catch (IOException e1) {
			result.setMessage(e1.getMessage());
		} catch (JsonSyntaxException e2) {
			String mensaje = result.getMessage();
			
			if (mensaje == null || mensaje.isEmpty()) {
				mensaje = e2.getMessage() + ". " + resp;
			}
			
			result.setMessage(mensaje);
		}
		
		return result;
	}

	public RespuestaEstatusSAT consultarEstadoSAT(String apikey, String uuid, String rfcEmisor,
			String rfcReceptor, String total) {
		RespuestaEstatusSAT result = new RespuestaEstatusSAT();
		String resp = "";
		try {
			String operacion = "consultarEstadoSAT";
			// SE ABRE LA CONEXIN
			mUrl = new URL(url+operacion);
			urlConn = (HttpURLConnection) mUrl.openConnection();

			// SE DEFINE LA CODIFICACIN DEL MENSAJE
			String charset = "UTF-8";  // Or in Java 7 and later, use the constant: java.nio.charset.StandardCharsets.UTF_8.name()

			// SE DEFINE EL CUERPO DEL MENSAJE
			String query = String.format("apikey=%s&uuid=%s&rfcEmisor=%s&rfcReceptor=%s&total=%s",
					URLEncoder.encode(apikey, charset),
				    URLEncoder.encode(uuid, charset),
				    URLEncoder.encode(rfcEmisor, charset),
				    URLEncoder.encode(rfcReceptor, charset),
				    URLEncoder.encode(total, charset)
				    );
			
			// SE REALIZA LA SOLICITUD
			resp = doRequest(query, charset);
			
			// SE SERIALIZA LA RESPUESTA A UN OBJETO
			result = new Gson().fromJson(resp, RespuestaEstatusSAT.class);
		} catch (IOException e1) {
			result.setCodigoEstatus(e1.getMessage());
		} catch (JsonSyntaxException e2) {
			String mensaje = e2.getMessage() + ". " + resp;
			result.setCodigoEstatus(mensaje);
		}
		
		return result;
	}

	public static String doRequest(String query, String charset) {
		String result = "";
		try {			
			// SE DEFINEN LOS HEADERS DEL MENSAJE
			urlConn.setDoOutput(true);
			urlConn.setRequestProperty("Accept-Charset", charset);
			urlConn.setRequestProperty("Content-Type", "application/x-www-form-urlencoded;charset=" + charset);
			
			// SE VERIFICA QUE EL CUERPO DEL MENSAJE NO SEA VACO
			if (query != null) {
				// SE REALIZA LA SOLICITUD
				urlConn.setRequestProperty("Content-Length", Integer.toString(query.length()));
				urlConn.getOutputStream().write(query.getBytes("UTF8"));
				// SE OBTIENE LA RESPUESTA
				InputStream response = urlConn.getInputStream();
				result = new BufferedReader(new InputStreamReader(response)).lines().collect(Collectors.joining("\n"));
				result = new String(result.getBytes(), "UTF8");
			}
		} catch (IOException e) {
			result = e.getMessage();
		}
		
		return result;
	}
}
