package api.facturalo.test;

/**
 * PARA MANEJO DE ARCHIVOS
 */
import java.io.File;
import java.io.IOException;
import java.nio.file.Files;
import java.util.Base64;

/**
 * PARA USO DEL API DE CONEXIN
 */
import api.facturalo.rest.Conexion;
import api.facturalo.rest.RespuestaCancelar;
import api.facturalo.rest.RespuestaEstatusSAT;
import api.facturalo.rest.RespuestaTimbrado;

public class MainTest {

	public static void main(String[] args) {
		/**
		 * @construct Conexion(endpoint, validateSSL)
		 * @param String  endpoint		-->	URL de pruebas  produccin
		 * @param boolean validateSSL	--> Validacin de certificado SSL
		 */
		Conexion cliente = new Conexion("https://dev.facturaloplus.com/api/rest/servicio/", true);
		String apikey = "93edc4af66b84c938f66a56ca0596205";
		
		try {
			/**
			 * * * * MENU * * * *
			 * 1) Timbrar (xml con sello)
			 * 2) Timbrar (xml sin sello)
			 * 3) Cancelar (por CSD)
			 * 4) Cancelar (por PFX)[+seguridad]
			 * 5) Consultar Estado en el SAT
			 */
			int opc = 5; 
			
			switch(opc)
			{
				case 1:
					{
						// SE CARGA String EN utf-8 DEL XML CON LA INFORMACIN DEL CFDI
						File pathIn = new File("src/api/facturalo/test/files/ejemplo_cfdi40-sinsello.xml");
						String cfdi = new String(Files.readAllBytes(pathIn.toPath()),"UTF8");
						
						RespuestaTimbrado response = cliente.timbrar(apikey, cfdi);
						
						System.out.println("Codigo: "+ response.getCode());
						System.out.println("Mensaje: "+ response.getMessage());
						System.out.println("CFDI: "+ response.getData());
					}
					break;
				case 2:
					{
						// SE CARGA String EN utf-8 DEL XML CON LA INFORMACIN DEL CFDI (SIN SELLO)
						File pathIn = new File("src/api/facturalo/test/files/ejemplo_cfdi40-sinsello.xml");
						String cfdi = new String(Files.readAllBytes(pathIn.toPath()),"UTF8");
						
						// SE CARGA String EN Base64 DEL .key DEL CSD DEL EMISOR
						pathIn = new File("src/api/facturalo/test/files/CSD_EKU9003173C9.key.pem");
						String keyPEM = new String(Files.readAllBytes(pathIn.toPath()));
						
						RespuestaTimbrado response = cliente.timbrarConSello(apikey, cfdi, keyPEM);
						
						System.out.println("Codigo: "+ response.getCode());
						System.out.println("Mensaje: "+ response.getMessage());
						System.out.println("CFDI: "+ response.getData());
					}
					break;
				case 3:
					{
						// SE CARGA String EN Base64 DEL CONTENIDO DE LA LLAVE PRIVADA (.key)
						File pathIn = new File("src/api/facturalo/test/files/CSD_EKU9003173C9.key");
						String keyCSD = new String(Base64.getEncoder().encodeToString( Files.readAllBytes(pathIn.toPath()) ) );
						
						// SE CARGA String EN Base64 DEL CONTENIDO DE LA LLAVE PRIVADA (.cer)
						pathIn = new File("src/api/facturalo/test/files/CSD_EKU9003173C9.cer");
						String cerCSD = new String(Base64.getEncoder().encodeToString( Files.readAllBytes(pathIn.toPath()) ) );
						
						String passCSD = "12345678a";
						String uuid = "dd6bfcea-5bde-418b-af06-90fa2ab54d58";
						String rfcEmisor = "EKU9003173C9";
						String rfcReceptor = "URE180429TM6";
						String total = "1.12";
						String motivo = "02";
						String foliosustitucion = "";
						
						RespuestaCancelar response = cliente.cancelar(apikey, keyCSD, cerCSD, passCSD,
								uuid, rfcEmisor, rfcReceptor, total, motivo, foliosustitucion);
						
						System.out.println("Codigo: "+ response.getCode());
						System.out.println("Mensaje: "+ response.getMessage());
						System.out.println("Acuse: "+ response.getData());
						System.out.println("Estatus: "+ response.getStatus());
					}
					break;
				case 4:
					{
						// SE CARGA String EN Base64 DEL CONTENIDO DEL ARCHIVO PFX QUE CONTIENE LA INFORMACIN ENCRIPTADA DEL CSD
						File pathIn = new File("src/api/facturalo/test/files/CSD_EKU9003173C9.pfx");
						String filePFX = new String(Base64.getEncoder().encodeToString( Files.readAllBytes(pathIn.toPath()) ) );
						
						String passPFX = "12345678a";
						String uuid = "dd6bfcea-5bde-418b-af06-90fa2ab54d58";
						String rfcEmisor = "EKU9003173C9";
						String rfcReceptor = "URE180429TM6";
						String total = "1.12";
						String motivo = "02";
						String foliosustitucion = "";
						
						RespuestaCancelar response = cliente.cancelarPFX(apikey, filePFX, passPFX,
								uuid, rfcEmisor, rfcReceptor, total, motivo, foliosustitucion);
						
						System.out.println("Codigo: "+ response.getCode());
						System.out.println("Mensaje: "+ response.getMessage());
						System.out.println("Acuse: "+ response.getData());
						System.out.println("Estatus: "+ response.getStatus());
					}
					break;
				case 5:
					{
						String uuid = "dd6bfcea-5bde-418b-af06-90fa2ab54d58";
						String rfcEmisor = "EKU9003173C9";
						String rfcReceptor = "URE180429TM6";
						String total = "1.12";
						
						RespuestaEstatusSAT response = cliente.consultarEstadoSAT(apikey, uuid, rfcEmisor, rfcReceptor, total);
						
						System.out.println("Codigo Estatus: "+ response.getCodigoEstatus());
						System.out.println("Es Cancelable: "+ response.getEsCancelable());
						System.out.println("Estado: "+ response.getEstado());
						System.out.println("Estatus Cancelacion: "+ response.getEstatusCancelacion());
					}
					break;
			}
		
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

}
